
package de.codedo.conflicteditor.gui;

import de.codedo.conflicteditor.Difference;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.util.List;

import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.table.TableModel;

import org.codehaus.jackson.JsonNode;

public class ConflictEditor
{
    private ConflictEditorController _controller;
    private ConflictEditorFrame _frame;

    public static void main(String[] args)
    {
        try
        {
            new ConflictEditor().run();
        }
        catch (Exception ex)
        {
            System.out.println("Uncaught toplevel Exception");
            ex.printStackTrace(System.out);
        }
    }

    private void run()
    {
        _controller = new ConflictEditorController();
        _frame = new ConflictEditorFrame();
        connectListeners();
        _frame.pack();
        _frame.setVisible(true);
    }

    private void connectListeners()
    {
        connectFindConflictsButton();
        connectConflictsList();
        connectDifferencesTable();
    }

    private void connectFindConflictsButton()
    {
        Executable<ActionEvent> executable = new Executable<ActionEvent>()
        {
            @Override
            public void execute(ActionEvent event) throws Exception
            {
                findConflicts();
            }
        };
        ExceptionHandlingActionListener listener = new ExceptionHandlingActionListener(executable);
        _frame.getFindConflictsButton().addActionListener(listener);
    }

    private void connectConflictsList()
    {
        Executable<ListSelectionEvent> executable = new Executable<ListSelectionEvent>()
        {
            @Override
            public void execute(ListSelectionEvent event) throws Exception
            {
                if (event.getValueIsAdjusting() == false)
                {
                    selectConflictDocument(event.getFirstIndex());
                }
            }
        };
        ExceptionHandlingListSelectionListener listener = new ExceptionHandlingListSelectionListener(
            executable);
        _frame.getConflictsList().addListSelectionListener(listener);
    }

    private void connectDifferencesTable()
    {
        Executable<ListSelectionEvent> executable = new Executable<ListSelectionEvent>()
        {
            @Override
            public void execute(ListSelectionEvent event) throws Exception
            {
                selectDifference();
            }
        };
        ExceptionHandlingListSelectionListener listener = new ExceptionHandlingListSelectionListener(
            executable);
        _frame.getDifferencesTable().getSelectionModel().addListSelectionListener(listener);
    }

    protected void findConflicts() throws IOException
    {
        String dbUrl = _frame.getDatabaseUrlTextField().getText();
        JsonNode conflicts = _controller.findConflicts(dbUrl);
        ListModel model = new ConflictsListModel(conflicts);
        _frame.getConflictsList().setModel(model);
    }

    protected void selectConflictDocument(int index) throws Exception
    {
        JsonNode conflictNode = (JsonNode)_frame.getConflictsList().getModel().getElementAt(index);
        List<Difference> differences = _controller.differencesForConflict(conflictNode);
        TableModel model = new DifferenceTableModel(differences);
        _frame.getDifferencesTable().setModel(model);
    }

    protected void selectDifference()
    {
        System.out.println("select a difference");
    }
}
