
package de.codedo.conflicteditor.gui;

import de.codedo.conflicteditor.Conflict;
import de.codedo.conflicteditor.ConflictsView;
import de.codedo.conflicteditor.CouchDb;
import de.codedo.conflicteditor.Difference;
import de.codedo.conflicteditor.DocumentMatcher;
import de.codedo.conflicteditor.UrlConnectionHttpAccess;

import java.io.IOException;
import java.io.Reader;
import java.net.URL;
import java.util.List;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.ObjectMapper;

public class ConflictEditorController extends Object
{
    private CouchDb _database;

    public JsonNode findConflicts(String dbUrl) throws IOException
    {
        _database = new CouchDb(dbUrl);
        ConflictsView conflictsView = new ConflictsView(_database);
        return conflictsView.getConflicts();
    }

    public List<Difference> differencesForConflict(JsonNode conflictNode) throws IOException
    {
        Conflict conflict = new Conflict(conflictNode);
        JsonNode currentDocument = conflict.getCurrentDocument();
        JsonNode conflictDocument = loadConflictingDocument(conflict);
        return new DocumentMatcher(currentDocument, conflictDocument).compare();
    }

    private JsonNode loadConflictingDocument(Conflict conflict) throws IOException
    {
        URL documentUrl = _database.urlForDocumentAndRevision(conflict.getId(),
            conflict.getConflictRevision());
        Reader reader = new UrlConnectionHttpAccess(documentUrl).get();
        return parseJson(reader);
    }

    private JsonNode parseJson(Reader reader) throws IOException
    {
        try
        {
            return new ObjectMapper().readTree(reader);
        }
        finally
        {
            reader.close();
        }
    }
}
